@@ -275,9 +275,9 @@ def compute_colors(data: Table) -> np.ndarray:
 
     Returns
     -------
-   Colors for each data instance and each feature. The shape of the matrix is
-   M x N x C, where M is a number of instances, N is a number of features, and
-   C is 3 (one value for each RGB channel).
+    Colors for each data instance and each feature. The shape of the matrix is
+    M x N x C, where M is a number of instances, N is a number of features, and
+    C is 3 (one value for each RGB channel).
     """
 
     def continuous_colors(x):
@@ -310,7 +310,7 @@ def compute_colors(data: Table) -> np.ndarray:
 
     # the final array is dense and we do not expect huge matrices here
     values = data.X.toarray() if sparse.issparse(data.X) else data.X
-    colors = np.zeros(values.shape + (3,))
+    colors = np.zeros(values.shape + (3,), dtype=float)
     is_discrete = np.array(
         [a.is_discrete for a in data.domain.attributes], dtype=bool
     )
@@ -322,7 +322,7 @@ def compute_colors(data: Table) -> np.ndarray:
     )
 
     colors[np.isnan(colors)] = 100
-    return colors
+    return colors.astype(int)
 
 
 def get_shap_values_and_colors(
