@@ -12,7 +12,7 @@ from Orange.data import Table, Domain, DiscreteVariable
 from Orange.data.table import DomainTransformationError
 from Orange.evaluation import CA, MSE, AUC
 from Orange.regression import RandomForestRegressionLearner, \
-    TreeLearner as TreeRegressionLearner
+    TreeLearner as TreeRegressionLearner, NNRegressionLearner
 
 from orangecontrib.explain.inspection import permutation_feature_importance, \
     _wrap_score, _check_model, individual_condition_expectation
@@ -123,19 +123,6 @@ class TestUtils(unittest.TestCase):
         mocked_model.predict.assert_not_called()
         self.assertAlmostEqual(baseline_score, 2, 0)
 
-    def test_remove_init_unlocked(self):
-        """
-        When this test starts to fail:
-        - remove code in
-        /Users/vesna/orange3-explain/orangecontrib/explain/__init__.py
-        - remove this test
-        - set minimum Orange version to 3.31.0
-        """
-        self.assertGreater(
-            "3.35.0",
-            pkg_resources.get_distribution("orange3").version
-        )
-
 
 class TestPermutationFeatureImportance(unittest.TestCase):
     @classmethod
@@ -308,7 +295,8 @@ class TestIndividualConditionalExpectation(unittest.TestCase):
         data = data.transform(Domain(data.domain.attributes, class_var))
         model1 = RandomForestLearner(n_estimators=10, random_state=0)(data)
 
-        data.Y = np.abs(data.Y - 1)
+        with data.unlocked(data.Y):
+            data.Y = np.abs(data.Y - 1)
         model2 = RandomForestLearner(n_estimators=10, random_state=0)(data)
 
         res = individual_condition_expectation(model1, data, data.domain[0])
@@ -332,6 +320,17 @@ class TestIndividualConditionalExpectation(unittest.TestCase):
         self.assertEqual(res["individual"].shape, (1, 506, 504))
         self.assertEqual(res["values"].shape, (504,))
 
+    def test_retain_original_values(self):
+        data = self.housing
+        nn = NNRegressionLearner(random_state=0)(data)
+        res_nn = individual_condition_expectation(nn, data, data.domain[0])
+        rf = RandomForestRegressionLearner(n_estimators=10, random_state=0)(data)
+        res_rf = individual_condition_expectation(rf, data, data.domain[0])
+        self.assertEqual(res_nn["values"].min(), res_rf["values"].min())
+        self.assertEqual(res_nn["values"].max(), res_rf["values"].max())
+        self.assertEqual(res_nn["values"].shape, res_rf["values"].shape)
+        self.assertEqual(len(set(res_nn["values"])), len(res_rf["values"]))
+
     def test_multi_class(self):
         data = self.iris
         model = RandomForestLearner(n_estimators=10, random_state=0)(data)
